/*
 * Copyright (c) 2008 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/reactive/).
 */
package org.xmloperator.reactive;

import org.xmloperator.reactive.exception.ActiveReactiveMachineException;

/**
 * This the place for obtaining a ReactiveMachine with its keys.
 * There at most one ReactiveMachine by Java Virtual Machine at a time.
 */
public interface ReactiveFactory {

	/**
	 * Creates an ReactiveMachine if there is not an active ReactiveMachine
	 *   at that time.
	 * There can be at more one active ReactiveMachine at a time.
	 * Provides the keys that allow to use the given ReactiveMachine.
	 * @param breathe_duration_ms the duration in millisecond of the breathe
	 *   periods.
	 * @param crashingReport for reporting agent crash.
	 * @param machineKeys is a recipient for receiving the keys of
	 *   the ReactiveMachine.
	 * @return the ReactiveMachine.
	 * @exception ActiveReactiveMachineException is throwed when there is already
	 * 	 an active ReactiveMachine.
	 */
	public ReactiveMachine newReactiveMachine(int breathe_duration_ms,
			CrashingReport crashingReport, MachineKeys machineKeys)
			throws ActiveReactiveMachineException;
	
	/**
	 * Returns the currently active ReactiveMachine if any, null elsewhere.
	 * This method doesn't provide the keys of the ReactiveMachine, if any.
	 * @return the currently active ReactiveMachine if any, null elsewhere.
	 */
	public ReactiveMachine getReactiveMachine();
}
