/*
 * Copyright (c) 2008 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/reactive/).
 */
package org.xmloperator.reactive.agent;

import java.io.PrintStream;

import org.xmloperator.reactive.Agent;
import org.xmloperator.reactive.Event;
import org.xmloperator.reactive.LaunchingContext;
import org.xmloperator.reactive.Message;
import org.xmloperator.reactive.ReactiveCommunity;
import org.xmloperator.reactive.ReactiveEntity;

/**
 * Agent implementation.
 */
public abstract class AgentImpl implements Agent {
	
	protected LaunchingContext launchingContext = null;
	protected ReactiveEntity reac = null;
	protected PrintStream trace = null;
	protected String agentId;
	
	public AgentImpl() {
		this.agentId = this.getClass().getName();
	}
	
	public void addParameter(long value) {
		StringBuffer sb = new StringBuffer(this.agentId);
		sb.append('/');
		sb.append(value);
		this.agentId = sb.toString();
	}
	
	public void addParameter(String value) {
		if (value.indexOf('"') > 0)
			throw new IllegalArgumentException();
		StringBuffer sb = new StringBuffer(this.agentId);
		sb.append('/');
		sb.append('"');
		sb.append(value);
		sb.append('"');
		this.agentId = sb.toString();
	}
	
	protected ReactiveCommunity newCommunity() {
		return null;
	}
	
	///////////////////////
	// Agent implementation
	///////////////////////
	
	public void init(LaunchingContext launchingContext) {
		this.launchingContext = launchingContext;
		this.reac = (ReactiveEntity)launchingContext.getArgumentValue(
				Agent.ARG_REACTIVE_ENTITY);
		this.trace = (PrintStream)launchingContext.getArgumentValue(
				Agent.ARG_TRACE_PRINT_STREAM);
		if (this.reac.getCommunity() == null) {
			ReactiveCommunity community =
				(ReactiveCommunity)launchingContext.getArgumentValue(
					Agent.ARG_DESTINATION_COMMUNITY);
			if (community == null) {
				community = this.newCommunity();
				if (community != null)
					launchingContext.addArgument(
							Agent.ARG_DESTINATION_COMMUNITY, community);
			}
			if (community != null)
				this.reac.connect(community);
		}
	}

	public final String getAgentId() {
		return this.agentId;
	}

	public final boolean isLaunched() {
		return this.reac != null;
	}
	
	public boolean continues() {
		return false;
	}

	public int idlePeriodDuration_ms() {
		return Agent.INFINITE_IDLE_PERIODE_DURATION;
	}
	
	public void endOfIdlePeriod() {
	}
	
	public boolean processEvent(Event event) {
		return false;
	}

	public boolean processMessage(Message message, Object attachment) {
		return false;
	}

	public void terminate() {
	}
	
	////////////////////////
	// Object implementation
	////////////////////////

	public String toString() {
		return this.reac != null ? this.reac.getEntityId() : super.toString();	
	}
}
