/*
 * Copyright (c) 2008 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/reactive/).
 */
package org.xmloperator.reactive.element;

/**
 * Element that can have child elements.
 */
public interface ParentElement extends Element {

	/**
	 * Returns the child count of this ParentElement.
	 * @return the child count of this ParentElement.
	 */
	public int getChildCount();

	/**
	 * Returns the first child of this ParentElement if any, null elsewhere.
	 * @return the first child of this ParentElement if any, null elsewhere.
	 */
	public Element getFirstChild();

	/**
	 * Returns the last child of this ParentElement if any, null elsewhere.
	 * @return the last child of this ParentElement if any, null elsewhere.
	 */
	public Element getLastChild();
	
	/**
	 * Returns the child with the given name.
	 * Returns null if there is no or more than one child with the given name.
	 * @param namespaceURI a namespace URI.
	 * @param tagName a tag name.
	 * @return the child with the given name.
	 */
	public Element getChild(String namespaceURI, String tagName);
	
	/**
	 * Returns the ChildrenConstraint, if any.
	 * @return the ChildrenConstraint, if any.
	 */
	public ChildrenConstraint getChildrenContraint();
}
