/*
 * Copyright (c) 2008 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/reactive/).
 */
package org.xmloperator.reactive.element.action;

import org.xmloperator.reactive.impl.ActionImpl;

import org.xmloperator.reactive.element.Element;
import org.xmloperator.reactive.element.ElementWritingAccess;

/**
 * Action applied to an Element.
 */
public abstract class ElementActionImpl extends ActionImpl
		implements ElementAction {

	public final ElementWritingAccess elementWritingAccess;
	public final Element referenceElement;
	
	public ElementActionImpl(ElementWritingAccess elementWritingAccess,
			Element referenceElement, int status) {
		super(status);
		if (elementWritingAccess == null || referenceElement == null)
			throw new IllegalArgumentException();
		this.elementWritingAccess = elementWritingAccess;
		this.referenceElement = referenceElement;
	}
	
	/**
	 * Returns true if this ElementAction is an ElementInsertAction.
	 * @return true if this ElementAction is an ElementInsertAction.
	 */
	public boolean isElementInsertAction() {
		return false;
	}
	
	/**
	 * Returns true if this ElementAction is an ElementAppendAction.
	 * @return true if this ElementAction is an ElementAppendAction.
	 */
	public boolean isElementAppendAction() {
		return false;
	}
	
	/**
	 * Returns true if this ElementAction is an ElementRemoveAction.
	 * @return true if this ElementAction is an ElementRemoveAction.
	 */
	public boolean isElementRemoveAction() {
		return false;
	}
	
	/**
	 * Returns true if this ElementAction is an ChildrenRemoveAction.
	 * @return true if this ElementAction is an ChildrenRemoveAction.
	 */
	public boolean isChildrenRemoveAction() {
		return false;
	}
	
	/**
	 * Returns true if this ElementAction is an ChildrenSortAction.
	 * @return true if this ElementAction is an ChildrenSortAction.
	 */
	public boolean isChildrenSortAction() {
		return false;
	}
	
	////////////////////////
	// Object implementation
	////////////////////////
	
	public String toString() {
		return super.toString() + this.referenceElement.hierarchicalPosition();
	}
}
