/*
 * Copyright (c) 2008 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/reactive/).
 */
package org.xmloperator.reactive.element.action.reject;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;

import org.xmloperator.reactive.element.Element;
import org.xmloperator.reactive.element.Rebuilder;
import org.xmloperator.reactive.element.TranslatableObject;

import org.xmloperator.reactive.element.spec.ElementSpecification;
import org.xmloperator.reactive.element.spec.ElementSpecRebuilderFactory;

/**
 * Rebuilder of a ElementAffectRejector.
 */
public final class ElementTouchRejectorRebuilder implements Rebuilder {

	private Rebuilder elementSpecRebuilder = null;
	private ElementSpecification elementSpecification = null;
	private String rejectionType = null;
	private boolean isRejectionTypeEnded = false;

	public ElementTouchRejectorRebuilder() {
	}
	
	///////////////////////////
	// Rebuilder implementation
	///////////////////////////
	
	public void startElement(
			String namespaceURI,
			String localName,
			String rawName,
			Attributes atts)
			throws SAXException {
		if (this.elementSpecRebuilder != null)
			this.elementSpecRebuilder.startElement(
				namespaceURI, localName, rawName, atts);
		else if (this.rejectionType == null) {
			if (localName != ElementTouchRejector.REJECT_MODIFY
				&& localName != ElementTouchRejector.REJECT_REMOVE)
				throw new SAXException(new IllegalStateException());
			this.rejectionType = localName;
		}
		else
			try {
				if (namespaceURI != Element.REACTIVE_NAMESPACE)
					throw new IllegalStateException();
				this.elementSpecRebuilder =
					ElementSpecRebuilderFactory.newElementSpecRebuilder(
						localName);
			}
			catch (Exception exception) {
				throw new SAXException(exception);
			}
	}

	public TranslatableObject endElement(
			String namespaceURI,
			String localName,
			String rawName)
			throws SAXException {
		if (this.rejectionType != null && !this.isRejectionTypeEnded) {
			this.isRejectionTypeEnded = true;
			return null;
		}
		if (this.elementSpecification != null) {
			if (namespaceURI != Element.REACTIVE_NAMESPACE
					|| !localName.equals("reject"))
				throw new SAXException(new IllegalStateException());
			return new ElementTouchRejector(
				this.elementSpecification,
				this.rejectionType);
		}
		if (this.elementSpecRebuilder == null)
			throw new SAXException(new IllegalStateException());
		this.elementSpecification =
			(ElementSpecification)this.elementSpecRebuilder.endElement(
				namespaceURI, localName, rawName);
		if (this.elementSpecification != null)
			this.elementSpecRebuilder = null;
		return null;
	}

	public void characters(char[] ch, int start, int length)
			throws SAXException {
		if (this.elementSpecRebuilder != null)
			this.elementSpecRebuilder.characters(ch, start, length);
		else
			throw new SAXException(new IllegalStateException());
	}
}
