/*
 * Copyright (c) 2008 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/reactive/).
 */
package org.xmloperator.reactive.element.facet;

import org.xmloperator.reactive.exception.ReactiveException;

import org.xmloperator.reactive.Action;
import org.xmloperator.reactive.ActionAttachment;
import org.xmloperator.reactive.Agent;
import org.xmloperator.reactive.LaunchingContext;
import org.xmloperator.reactive.Message;
import org.xmloperator.reactive.ReactiveMachine;

import org.xmloperator.reactive.facet.FacetImpl;

import org.xmloperator.reactive.element.Element;
import org.xmloperator.reactive.element.ObjectDataElement;
import org.xmloperator.reactive.element.ParentElement;

import org.xmloperator.reactive.element.action.ElementAction;

import org.xmloperator.reactive.element.action.reject.ActionRejector;

/**
 * This facet processes the proposal actions that are became executable.
 * It generates the action message if the action is not rejected
 *   by some ActionRejector.
 */
public final class ActionExecutionFacet extends FacetImpl {
	
	private static boolean isActionRejected(ElementAction elementAction) {
		ParentElement touchedParentElement =
			elementAction.getTouchedParentElement();
		while (true) {
			ParentElement rejectsElement =
				(ParentElement)touchedParentElement.getChild(
						Element.REACTIVE_NAMESPACE, "rejects");
			if (rejectsElement != null
					&& isActionRejected(elementAction, rejectsElement))
				return true;
			touchedParentElement = touchedParentElement.getParent();
			if (touchedParentElement == null)
				return false;
		}
	}
	
	private static boolean isActionRejected(
			Action action, ParentElement rejectsElement) {
		ObjectDataElement rejectElement =
			(ObjectDataElement)rejectsElement.getFirstChild();
		while (rejectElement != null) {
			ActionRejector actionRejector =
				(ActionRejector)rejectElement.getData();
			if (actionRejector.rejects(action))
				return true;
			rejectElement =
				(ObjectDataElement)rejectElement.getNextSibling(); 
		}
		return false;
	}
	
	public ActionExecutionFacet() {
	}
	
	////////////////////////////
	// Facet implementation
	////////////////////////////

	public void init(LaunchingContext launchingContext) {
		super.init(launchingContext);
		{
			String agentProfile = (String)launchingContext.getArgumentValue(
					Agent.ARG_AGENT_PROFILE);
			if (agentProfile.indexOf(ReactiveMachine.PROFILE_ACTION_EXECUTION)
					< 0)
				throw new ReactiveException(
			"This agent has not the profile for generating executable actions");
		}
	}
	
	public void processActionMessage(
			Message message, ActionAttachment attachment) {
		Action action = attachment.getAction();
		if (action.getStatus() == Action.STATUS_EXECUTABLE
				&& (action instanceof ElementAction)) {
			if (isActionRejected((ElementAction)action)) {
				if (this.trace != null)
					this.trace.println(
						attachment.getSenderEntityId()
						+ " proposal " + action + " is rejected");
			}
			else
				this.reac.generateMessage(message, attachment);
		}
	}
}
