/*
 * Copyright (c) 2008 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/reactive/).
 */
package org.xmloperator.reactive.element.spec;

import org.xmloperator.reactive.element.Element;
import org.xmloperator.reactive.element.ParentElement;

import org.xmloperator.reactive.element.impl.ElementImpl;
import org.xmloperator.reactive.element.impl.ParentElementImpl;

/**
 * ElementSpecification based on the name of the Element.
 */
public final class ElementNameSpecification
		implements ElementSpecification {
	
	public final String namespaceURI;
	public final String localName;
	
	public ElementNameSpecification(String namespaceURI, String localName) {
		this.namespaceURI =
			namespaceURI != null ? namespaceURI.intern() : null;
		this.localName = localName;
	}
	
	//////////////////////////////////////
	// ElementSpecification implementation
	//////////////////////////////////////
	
	public boolean accepts(Element element) {
		return element.getNamespaceURI() == this.namespaceURI
			&& element.getLocalName().equals(this.localName);
	}
	
	////////////////////////////////////
	// TranslatableObject implementation
	////////////////////////////////////

	public ParentElement translate() {
		return new ParentElementImpl(Element.REACTIVE_NAMESPACE, "name",
				new Element[] {
					new ElementImpl(this.namespaceURI, this.localName)});
	}

	public Class getRebuilderClass() {
		return ElementNameSpecRebuilder.class;
	}
}
