/*
 * Copyright (c) 2008 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/reactive/).
 */
package org.xmloperator.reactive.element.spec;

import org.xmloperator.reactive.element.Element;
import org.xmloperator.reactive.element.ParentElement;

import org.xmloperator.reactive.element.impl.ParentElementImpl;

/**
 * ElementSpecification generalized to all descendants.
 */
public final class IncludingDescendanceElementSpecification
		implements ElementSpecification {
	
	public final ElementSpecification elementSpecification;
	
	public IncludingDescendanceElementSpecification(
			ElementSpecification elementSpecification) {
		this.elementSpecification = elementSpecification;
	}
	
	//////////////////////////////////////
	// ElementSpecification implementation
	//////////////////////////////////////
	
	public boolean accepts(Element element) {
		do {
			if (this.elementSpecification.accepts(element))
				return true;
			element = element.getParent();
		}
		while (element != null);
		return false;
	}
	
	////////////////////////////////////
	// TranslatableObject implementation
	////////////////////////////////////

	public ParentElement translate() {
		return new ParentElementImpl(
				Element.REACTIVE_NAMESPACE, "includingDescendance",
				new Element[] {this.elementSpecification.translate()});
	}

	public Class getRebuilderClass() {
		return IncludingDescendanceElementSpecRebuilder.class;
	}
}
