/*
 * Copyright (c) 2008 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/reactive/).
 */
package org.xmloperator.reactive.facet;

import java.util.Map;
import java.util.HashMap;

import org.xmloperator.reactive.exception.BadCharInNameException;

import org.xmloperator.reactive.Message;
//import org.xmloperator.reactive.LaunchingContext;
import org.xmloperator.reactive.ReactiveCommunity;
import org.xmloperator.reactive.Terms;
import org.xmloperator.reactive.TermsAttachment;

import org.xmloperator.reactive.util.CommunityInstant;
import org.xmloperator.reactive.util.TermsAttachmentFactory;

/**
 * This facet collects data about meeted agents.
 * On "search" demand about an Agent, this one responds
 *  - if the searched agent was meeted in another community,
 *  - or if it is the searched agent.
 * The protocol of the "agent" service is the following :
 *   accept(search (agent [target:agentId]))
 *    -> emit(publish (agent [target:agentId] [communityInstant]))
 *    or -> emit(publish (agent))
 */
public final class AgentCatalogFacet extends FacetImpl {

	private static final String SEARCH = "search".intern();
	private static final String PUBLISH = "publish".intern();
	
	private static final String AGENT = "agent".intern();

	static {
		BadCharInNameException.verifyName(SEARCH);
		BadCharInNameException.verifyName(PUBLISH);
		
		BadCharInNameException.verifyName(AGENT);
	}
	
	private final Map fromAgentIdToCommunityInstant = new HashMap();
	
	public AgentCatalogFacet() {
	}
	
	public CommunityInstant getLastCommunityInstant(String agentId) {
		return
			(CommunityInstant)this.fromAgentIdToCommunityInstant.get(agentId);
	}
	
	private void notePresence(String agentId) {
		if (agentId == this.reac.getEntityId())
			return;
		ReactiveCommunity community = this.reac.getCommunity();
		long instant = community.getInstant();
		CommunityInstant communityInstant =
			(CommunityInstant)this.fromAgentIdToCommunityInstant.get(agentId);
		if (communityInstant != null)
			communityInstant.setCommunityInstant(community, instant);
		else {
			if (this.trace != null)
				this.trace.println(this.reac.getEntityId() + " discovers "
						+ agentId);
			this.fromAgentIdToCommunityInstant.put(
				agentId, new CommunityInstant(community, instant));
		}
	}
	
	private boolean process_search_message(TermsAttachment attachment) {
		Terms terms = attachment.getTerms();
		if (terms.count() != 2)
			return false;
		Object service = terms.get(0);
		if (service == AGENT) {
			String thisAgentId = this.reac.getEntityId();
			String targetAgentId = (String)terms.get(1);
			Message m_publish =
				this.reac.searchMessage(true, PUBLISH);
			if (targetAgentId == thisAgentId)
				this.reac.generateMessage(m_publish,
						TermsAttachmentFactory.newTermsAttachment(AGENT));
			else {
				CommunityInstant communityInstant =
					(CommunityInstant)this.fromAgentIdToCommunityInstant.get(
						targetAgentId);
				if (communityInstant != null
						&& communityInstant.getCommunity()
						!= this.reac.getCommunity())
					this.reac.generateMessage(m_publish,
							TermsAttachmentFactory.newTermsAttachment(
									AGENT, targetAgentId, communityInstant));
			}
			return true;
		}
		return false;
	}

	////////////////////////////
	// Facet implementation
	////////////////////////////

	public Object getService() {
		return AGENT;
	}
	
	public boolean processServiceMessage(String messageName,
			TermsAttachment attachment) {
		this.notePresence(attachment.getSenderEntityId());
		if (messageName == SEARCH) {
			this.process_search_message(attachment);
			return true;
		}
		return false;
	}

	public void processServiceExcludedMessage(String message,
			TermsAttachment attachment) {
		this.notePresence(attachment.getSenderEntityId());
	}
}
