/*
 * Copyright (c) 2008 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/reactive/).
 */
package org.xmloperator.reactive.impl;

import org.xmloperator.reactive.Action;
import org.xmloperator.reactive.exception.ReactiveException;

/**
 * Action implementation.
 * This implementation has no effect.
 */
public class ActionImpl implements Action {
	
	private int status;
	
	public ActionImpl(int status) {
		if (status != Action.STATUS_PROPOSED
				&& status != Action.STATUS_EXECUTABLE)
			throw new IllegalArgumentException();
		this.status = status;
	}
	
	void setStatus(int status) {
		boolean isOk = false;
		switch (this.status) {
			case Action.STATUS_PROPOSED:
				isOk = (status == Action.STATUS_EXECUTABLE);
				break;
			case Action.STATUS_EXECUTABLE:
				isOk = (status == Action.STATUS_EXECUTED);
				break;
		}
		if (!isOk)
			throw new IllegalStateException();
		this.status = status;
	}
	
	////////////////////////
	// Action implementation
	////////////////////////

	public final int getStatus() {
		return this.status;
	}
	
	public boolean equals(Action action) {
		return action instanceof ActionImpl;
	}

	public void execute(long key) throws ReactiveException {
	}

	public Action reverseAction(int status) {
		return new ActionImpl(status);
	}
	
	////////////////////////
	// Object implementation
	////////////////////////
	
	public String toString() {
		return this.status == Action.STATUS_PROPOSED ? "P" :
			this.status == Action.STATUS_EXECUTABLE ? "E" : "X";
	}
}
