/*
 * Copyright (c) 2008 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/reactive/).
 */
package org.xmloperator.reactive.impl;

import org.xmloperator.reactive.Event;
import org.xmloperator.reactive.Message;

/**
 * Reporter of the activity of a ReactiveCommunity.
 */
public interface ReactiveReporter {

	/**
	 * Reports the begin of an instant.
	 * @param instant : the reported instant.
	 */
	public void reportInstantBegin(long instant);

	/**
	 * Reports the generation of a base Event.
	 * @param event : the reported Event.
	 */
	public void reportBaseEventGeneration(Event event);
	
	/**
	 * Reports the generation of a Message.
	 * The Message doesn't have yet attachment.
	 * @param message : the reported Message.
	 */
	public void reportMessageGeneration(Message message);
	
	/**
	 * Reports the attachment of an Object to a Message.
	 * The Message just obtain a new attachment.
	 * @param message : the reported Message.
	 */
	public void reportMessageAttachment(Message message);
	
	/**
	 * Reports the end of an instant.
	 * @param instantHasBeenIdle : <tt>true</tt> if the instant has been idle for a part.
	 * @param broadcastCount : the count of broadcasting during the instant.
	 * @param broadcastedEventCount : the count of broadcasted events during the instant.
	 */
	public void reportInstantEnd(boolean instantHasBeenIdle,
		int broadcastCount, int broadcastedEventCount);
}
